@@ -228,27 +228,45 @@ pub fn map(attrs: TokenStream, item: TokenStream) -> TokenStream {
             #item
         }
     };
+
+    let mut tc_compatible = false;
     let mut key_type: Option<GenericArgument> = None;
     let mut value_type: Option<GenericArgument> = None;
     if let Type::Path(path) = *static_item.ty {
         if let Some(seg) = path.path.segments.last() {
+            let map_type_name = seg.ident.to_string();
             if let PathArguments::AngleBracketed(bracket) = &seg.arguments {
                 // <K, V> or <V>
-                let map_type_name = seg.ident.to_string();
-                match map_type_name.as_ref() {
+                match map_type_name.as_str() {
                     "Array" | "PerCpuArray" => {
                         if bracket.args.len() == 1 {
                             key_type = Some(parse_quote!(u32));
                             value_type = Some(bracket.args.first().unwrap().clone());
                         }
                     }
-                    "HashMap" => {
+                    "HashMap" | "PerCpuHashMap" | "LruHashMap" | "LruPerCpuHashMap"
+                    | "TcHashMap" => {
                         if bracket.args.len() == 2 {
                             key_type = Some(bracket.args.first().unwrap().clone());
                             value_type = Some(bracket.args.last().unwrap().clone());
                         }
                     }
-                    _ => {}
+                    "PerfMap" => {}
+                    _ => {
+                        panic!("unknown map type name: {}", map_type_name);
+                    }
+                }
+
+                if map_type_name == "TcHashMap" {
+                    tc_compatible = true;
+                }
+            } else {
+                // without generic types
+                match map_type_name.as_str() {
+                    "StackTrace" | "SockMap" | "ProgramArray" | "DevMap" => {}
+                    _ => {
+                        panic!("unknown map type name: {}", map_type_name);
+                    }
                 }
             }
         }
@@ -258,27 +276,62 @@ pub fn map(attrs: TokenStream, item: TokenStream) -> TokenStream {
         let mod_ident = syn::Ident::new(&mod_name, static_item.ident.span());
         let ktype = key_type.unwrap();
         let vtype = value_type.unwrap();
+        // CAUTION: When you change the names (MAP_BTF_XXXX and
+        // MAP_VALUE_ALIGN_XXXX) you should consider changing corresponding
+        // parts that use them.
         let map_btf_name = format!("MAP_BTF_{}", static_item.ident.to_string());
         let map_btf_ident = syn::Ident::new(&map_btf_name, static_item.ident.span());
-        tokens.extend(quote! {
-            mod #mod_ident {
-                #[allow(unused_imports)]
-                use super::*;
-                use core::mem;
+        let value_align_name = format!("MAP_VALUE_ALIGN_{}", static_item.ident.to_string());
+        let value_align_ident = syn::Ident::new(&value_align_name, static_item.ident.span());
+        if tc_compatible {
+            let btf_type_name = format!("____btf_map_{}", static_item.ident.to_string());
+            let btf_map_type = syn::Ident::new(&btf_type_name, static_item.ident.span());
+            tokens.extend(quote! {
+                mod #mod_ident {
+                    #[allow(unused_imports)]
+                    use super::*;
+                    use core::mem::{self, MaybeUninit};
 
-                #[repr(C)]
-                struct MapBtf {
-                    key_type: #ktype,
-                    value_type: #vtype,
+                    #[no_mangle]
+                    static #value_align_ident: MaybeUninit<#vtype> = MaybeUninit::uninit();
+
+                    #[repr(C)]
+                    struct #btf_map_type {
+                        key: #ktype,
+                        value: #vtype,
+                    }
+                    // `impl Sync` is needed to allow pointer types of keys and values
+                    unsafe impl Sync for #btf_map_type {}
+                    const N: usize = mem::size_of::<#btf_map_type>();
+                    #[no_mangle]
+                    #[link_section = "maps.ext"]
+                    static #map_btf_ident: #btf_map_type = unsafe { mem::transmute::<[u8; N], #btf_map_type>([0u8; N]) };
                 }
-                // `impl Sync` is needed to allow pointer types of keys and values
-                unsafe impl Sync for MapBtf {}
-                const N: usize = mem::size_of::<MapBtf>();
-                #[no_mangle]
-                #[link_section = "maps.ext"]
-                static #map_btf_ident: MapBtf = unsafe { mem::transmute::<[u8; N], MapBtf>([0u8; N]) };
-            }
-        });
+            });
+        } else {
+            tokens.extend(quote! {
+                mod #mod_ident {
+                    #[allow(unused_imports)]
+                    use super::*;
+                    use core::mem::{self, MaybeUninit};
+
+                    #[no_mangle]
+                    static #value_align_ident: MaybeUninit<#vtype> = MaybeUninit::uninit();
+
+                    #[repr(C)]
+                    struct MapBtf {
+                        key_type: #ktype,
+                        value_type: #vtype,
+                    }
+                    // `impl Sync` is needed to allow pointer types of keys and values
+                    unsafe impl Sync for MapBtf {}
+                    const N: usize = mem::size_of::<MapBtf>();
+                    #[no_mangle]
+                    #[link_section = "maps.ext"]
+                    static #map_btf_ident: MapBtf = unsafe { mem::transmute::<[u8; N], MapBtf>([0u8; N]) };
+                }
+            });
+        }
     }
     tokens.into()
 }
